@@ -60,12 +60,35 @@ my $setters = {
             );
         }
     },
+    traces => sub {
+        my ($setting, $traces) = @_;
+        $Carp::Verbose = $traces ? 1 : 0;
+    },
 };
 
+my $normalizers = {
+    charset => sub {
+        my ($setting, $charset) = @_;
+        $charset = 'UTF-8' if $charset =~ /utf8/i;
+        return $charset;
+    },
+};
+
+sub normalize_setting {
+    my ($class, $setting, $value) = @_;
+    $value = $normalizers->{$setting}->($setting, $value) 
+        if exists $normalizers->{$setting};
+    return $value;
+}
+
 # public accessor for get/set
 sub setting {
     my ($setting, $value) = @_;
 
+    # normalize the value if needed
+    $value = Dancer::Config->normalize_setting($setting, $value)
+        if @_ == 2;
+
     # run the hook if setter
     $setters->{$setting}->(@_)
       if (@_ == 2) && defined $setters->{$setting};
@@ -127,7 +150,8 @@ sub load_settings_from_yaml {
     my $config = YAML::LoadFile($file)
       or confess "Unable to parse the configuration file: $file";
 
-    @{$SETTINGS}{keys %$config} = values %$config;
+    setting($_, $config->{$_}) for (keys %{ $config });
+
     return scalar(keys %$config);
 }
 
@@ -142,7 +166,8 @@ sub load_default_settings {
     $SETTINGS->{apphandler}   ||= $ENV{DANCER_APPHANDLER}   || 'Standalone';
     $SETTINGS->{warnings}     ||= $ENV{DANCER_WARNINGS}     || 0;
     $SETTINGS->{auto_reload}  ||= $ENV{DANCER_AUTO_RELOAD}  || 0;
-    $SETTINGS->{environment}
+    $SETTINGS->{traces}       ||= $ENV{DANCER_TRACES}       || 0;
+    $SETTINGS->{environment} 
       ||= $ENV{DANCER_ENVIRONMENT}
       || $ENV{PLACK_ENV}
       || 'development';
@@ -244,6 +269,11 @@ a matching template in the directory $appdir/views/layout.
 
 If set to true, tells Dancer to consider all warnings as blocking errors.
 
+=head2 traces (boolean)
+
+If set to true, Dancer will display full stack traces when a warning or a die
+occurs. (Internally sets Carp::Verbose). Default to false.
+
 =head2 log (enum)
 
 Tells which log messages should be actullay logged. Possible values are
@@ -267,7 +297,7 @@ $public/$error_code.html if it exists.
 
 =head2 auto_reload (boolean)
 
-Requires L<Module::Refresh>.
+Requires L<Module::Refresh> and L<Clone>.
 
 If set to true, Dancer will reload the route handlers whenever the file where
 they are defined is changed. This is very useful in development environment but
